#pragma once

#ifndef CGSTVIDEO_H_INCLUDED
#define CGSTVIDEO_H_INCLUDED

#include <gst/gst.h>
#include "gstappsink.h"

//            gstreamer
static void GstVideoOnError(GstBus *bus, GstMessage *msg, void *pThis);

//       gstreamer
static void GstVideoOnStateChanged(GstBus *bus, GstMessage *msg, void *pThis);

//   , ,  appsink  ,        gstreamer,   
static GstFlowReturn GstVideoOnNewBuffer(GstAppSink *elt, void *pThis);

//   , ,  appsink  ,     ,      
static GstFlowReturn GstVideoOnNewPreroll(GstAppSink *elt, void *pThis);

//   , ,     
static void GstVideoOnEOS(GstAppSink *elt, void *pThis);

//       gstreamer.        gstreamer   
UINT GstVideoInit (LPVOID pParam);

class CGstVideo
{
public:
	CGstVideo(const char *pipelineString, const char *gstreamerPath);
	~CGstVideo();

	//   ?
	bool isPlaying();

	//   
	void start();

	//   
	void stop();

	//        gstreamer
	void onError(GstMessage *msg);

	//       gstreamer
	void onStateChanged(GstMessage *msg);

	// ,  appsink  ,        gstreamer,   
	GstFlowReturn onNewBuffer();

	// ,  appsink  ,     ,      
	GstFlowReturn onNewPreroll();

	// ,     
	void onEOS();

	//       gstreamer.        gstreamer   
	UINT init();

	//  
	void release();

	//      
	gint64 getPositionByMsec();

	//      
	gint64 getPositionByFrames();

	//      
	gint64 getPositionByPercent();

	//    
	gint getWidth();

	//    
	gint getHeight();

	//  FPS 
	gint getFPS();

	//     
	gint getQueueLenght();

	//     
	gint getChannels();

	//     
	char *getFrameBuffer();

	//    
	gchar *getInfoMessage();

	//    
	void cleartInfoMessage();

	//     
	gchar *getErrorMessage();

	//      
	void cleartErrorMessage();

protected:
	//   
	GstCaps *pBufferGstCaps;

	//    
	void setErrorMessage(const gchar *message);

	//   
	void setInfoMessage(const gchar *message);

private:
	//  pipeline  gstreamer
	gchar *pipelineString;

	//   gstreamer
	gchar *gstreamerPath;

	//  
	GstElement *pipeline;

	//  GLib       gstreamer
	GMainContext *pGMainContext;

	// GLib 
	GMainLoop *pGMainLoop;

	//  ,     appsink
	GstAppSink *pGstAppSink;

	//  
	GstBuffer *pGstBuffer;

	//  
	char *pFrameBuffer;

	//   
	gchar *errorMessageString;

	//  
	gchar *infoMessageString;

	//         
	int strcasecmp(const char *s1, const char *s2);
};

#endif // CGSTVIDEO_H_INCLUDED